import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.mortbay.jetty.Server;
import org.mortbay.jetty.servlet.Context;
import org.mortbay.jetty.servlet.ServletHolder;

/**
 * Launches an embedded Jetty Server and handles
 * configuration options.
 *
 */
public class CSMConfigServlet extends HttpServlet
{

  private Properties props = new Properties();
  private Server server;
  private Context context;
  public boolean isWorking = true;
  private File[] videoDirs;
  private File currentDir;
  private String GREEN = "#22AA22";
  private String RED = "#AA2222";
  private String BLUE = "#2222AA";
  private String BLACK = "#000000";

  public CSMConfigServlet()
  {
    //Set the location of the java.exe in the wrapper properties file.
    String javaPath = System.getProperty("sun.boot.library.path", "C:\\windows\\system32");
    javaPath += "\\java.exe";
    System.out.println(javaPath);
    
    String wrapperConfigFile = "./data/wrapper.conf";
    Properties wrapperProps = loadProperties(wrapperConfigFile);
    wrapperProps.setProperty("wrapper.java.command", javaPath);
    savePropertiesViaWriter(wrapperProps, wrapperConfigFile);
    
    //Configure csm.properties
    props = loadProperties("csm.properties");
    server = new Server(8181);
    context = new Context(server,"/",Context.SESSIONS);
    context.addServlet(new ServletHolder(this), "/*");
    try
    {
      server.start();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
    while(!server.isStarted())
    {
      //Waiting
    }
  }

  protected void doGet(HttpServletRequest request, HttpServletResponse response)
      throws ServletException, IOException
  {
    if(request.getPathInfo().equals("/Config")
       || request.getPathInfo().equals("/configVideoDirs")
       || request.getPathInfo().equals("/"))
    {
      renderConfigVideoDirs(request, response);
    }

    if(request.getPathInfo().equals("/configFileExtensions"))
    {
      renderConfigFileExtensions(request, response);
    }

    if(request.getPathInfo().equals("/configCleanupExtensions"))
    {
      renderConfigCleanupExtensions(request, response);
    }

    if(request.getPathInfo().equals("/configComskipLocation"))
    {
      renderConfigComskipLocation(request, response);
    }

    if(request.getPathInfo().equals("/configComskipConfigLocation"))
    {
      renderConfigConfigLocation(request, response);
    }

    if(request.getPathInfo().equals("/configCheckTime"))
    {
      renderConfigCheckTime(request, response);
    }

    if(request.getPathInfo().equals("/configExcludeHours"))
    {
      renderConfigExcludeHours(request, response);
    }

    if(request.getPathInfo().equals("/finish"))
    {
      if(props.getProperty("file.directories").trim().equals(""))
      {
        request.setAttribute("error", "You must specify at least one directory.");
        renderConfigVideoDirs(request, response);
      }
      else
      {
        saveProperties(props, "csm.properties");
        renderFinish(request, response);
        try
        {
          server.stop();
        }
        catch(Exception e)
        {
          e.printStackTrace();
        }
        isWorking = false;
      }
      
    }

    if(request.getPathInfo().equals("/SelectSubDirectory"))
    {
      String indexParam = request.getParameter("index");
      currentDir = new File(indexParam);
      videoDirs = currentDir.listFiles(new DirFilter());
      
      renderConfigVideoDirs(request, response);
    }

    if(request.getPathInfo().equals("/SelectParentDirectory"))
    {
      if(currentDir.getParentFile() != null)
      {
        currentDir = currentDir.getParentFile();
        videoDirs = currentDir.listFiles(new DirFilter());
      }
      else
      {
        currentDir = null;
        videoDirs = null;
      }
      
      renderConfigVideoDirs(request, response);
    }

    if(request.getPathInfo().equals("/SelectRootDirectory"))
    {
      currentDir = null;
      videoDirs = null;
      renderConfigVideoDirs(request, response);
    }

    if(request.getPathInfo().equals("/RemoveVideoDir"))
    {
      String indexParam = request.getParameter("index");
      String fileDirsProp = props.getProperty("file.directories");
      StringTokenizer tokens = new StringTokenizer(fileDirsProp, ",");
      int index = 1;
      boolean isFirst = true;
      String newProp = "";
      while (tokens.hasMoreTokens())
      {
        String fileDir = tokens.nextToken().trim();
        if(index != Integer.parseInt(indexParam))
        {
          if(isFirst)
          {
            isFirst = false;
          }
          else
          {
            newProp += ", ";
          }
          newProp += fileDir;
        }
        index++;
      }
      props.setProperty("file.directories", newProp);
      renderConfigVideoDirs(request, response);
    }

    if(request.getPathInfo().equals("/RemoveProcessExtension"))
    {
      String indexParam = request.getParameter("index");
      String prop = props.getProperty("process.file.extensions");
      StringTokenizer tokens = new StringTokenizer(prop, ",");
      int index = 1;
      boolean isFirst = true;
      String newProp = "";
      while (tokens.hasMoreTokens())
      {
        String ext = tokens.nextToken().trim();
        if(index != Integer.parseInt(indexParam))
        {
          if(isFirst)
          {
            isFirst = false;
          }
          else
          {
            newProp += ", ";
          }
          newProp += ext;
        }
        index++;
      }
      props.setProperty("process.file.extensions", newProp);
      renderConfigFileExtensions(request, response);
    }

    if(request.getPathInfo().equals("/RemoveCleanupExtension"))
    {
      String indexParam = request.getParameter("index");
      String prop = props.getProperty("cleanup.file.extensions");
      StringTokenizer tokens = new StringTokenizer(prop, ",");
      int index = 1;
      boolean isFirst = true;
      String newProp = "";
      while (tokens.hasMoreTokens())
      {
        String ext = tokens.nextToken().trim();
        if(index != Integer.parseInt(indexParam))
        {
          if(isFirst)
          {
            isFirst = false;
          }
          else
          {
            newProp += ", ";
          }
          newProp += ext;
        }
        index++;
      }
      props.setProperty("cleanup.file.extensions", newProp);
      renderConfigCleanupExtensions(request, response);
    }
    
    if(request.getPathInfo().equals("/RemoveComskipParam"))
    {
      String indexParam = request.getParameter("index");
      String prop = props.getProperty("comskip.parameters");
      StringTokenizer tokens = new StringTokenizer(prop, ",");
      int index = 1;
      boolean isFirst = true;
      String newProp = "";
      while (tokens.hasMoreTokens())
      {
        String param = tokens.nextToken().trim();
        if(index != Integer.parseInt(indexParam))
        {
          if(isFirst)
          {
            isFirst = false;
          }
          else
          {
            newProp += ", ";
          }
          newProp += param;
        }
        index++;
      }
      props.setProperty("comskip.parameters", newProp);
      renderConfigComskipLocation(request, response);
    }

    if(request.getPathInfo().equals("/RemoveRelatedExtension"))
    {
      String indexParam = request.getParameter("index");
      String prop = props.getProperty("related.file.extensions");
      StringTokenizer tokens = new StringTokenizer(prop, ",");
      int index = 1;
      boolean isFirst = true;
      String newProp = "";
      while (tokens.hasMoreTokens())
      {
        String ext = tokens.nextToken().trim();
        if(index != Integer.parseInt(indexParam))
        {
          if(isFirst)
          {
            isFirst = false;
          }
          else
          {
            newProp += ", ";
          }
          newProp += ext;
        }
        index++;
      }
      props.setProperty("related.file.extensions", newProp);
      renderConfigCleanupExtensions(request, response);
    }

  }

  private void renderConfigVideoDirs(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    PrintWriter writer = response.getWriter();
    printHeaderHtml(writer);
    //Print errors
    if(request.getAttribute("error") != null)
    {
      writer.println("<font color=\"#FF7777\">" + request.getAttribute("error") + "</font><br/><br/>");
    }

    //Video Dirs
    writer.println("<form name=\"vidForm\" action=\"AddVideoDir\" method=\"post\">");
    writer.println("<strong>Video Directories:</strong><br/>");
    writer.println("<em>Please choose the file directories where your videos are" +
    " located by selecting the checkbox next to them below, and then click the 'Add Video Dir(s)' button.</em><br/>");
    writer.println("<em>Click \"remove\" next to a selected directory to remove it.</em><br/>");
    writer.println("<em>Click on an available directory to access its sub-directories.</em><br/><br/>");

    String fileDirsProp = props.getProperty("file.directories");
    StringTokenizer tokens = new StringTokenizer(fileDirsProp, ",");
    writer.println("<table border=\"0\" cellpadding=\"10\" ><tr><td valign=\"top\">");
    writeColor(BLUE, writer, "<strong>Selected Directories</strong><br/>");
    
    if(tokens.countTokens() <= 0)
    {
      writeColor(RED, writer, "<strong>None Selected.</strong><br/>");
    }
    int index = 1;
    while (tokens.hasMoreTokens())
    {
      File fileDir = new File(tokens.nextToken().trim());
      writeColor(GREEN, writer,  "<strong>" + index + ") " + fileDir.getAbsolutePath() + "</strong>" + " &nbsp;&nbsp;&nbsp;<a href=\"RemoveVideoDir?index="
          + index  + "\">Remove</a><br/>");
      index++;
    }
    //writer.println("<input type=\"text\" name=\"videoDir\"/> <button type=\"submit\" name=\"dirButton\" value=\"Add Directory\">Add Directory</button>");

    writer.println("<br/>");
    writer.println("</td><td width=\"60\">&nbsp;</td><td>");
    
    writeColor(BLUE, writer, "<strong>Available Directories</strong><br/>");
    
    boolean isRootDirs;
    if(videoDirs == null || currentDir == null)
    {
      writer.println("<strong>Root</strong>");
      writer.println("<br/>");
      videoDirs = File.listRoots();
      isRootDirs = true;
    }
    else
    {
      isRootDirs = false;
      writer.println("<strong>" + currentDir.getAbsolutePath() + "</strong>");
      writer.println("<br/>");
      if(currentDir.getAbsolutePath().length() > 3)
      {
        writer.println("<a href=\"SelectRootDirectory\">.(Go to root directory)</a><br/>");
        writer.println("<a href=\"SelectParentDirectory\">..(Go up 1 directory)</a><br/>");
      }
      else
      {
        writer.println("<a href=\"SelectRootDirectory\">.(Go to root directory)</a><br/>");
      }
    }
    for(int i = 0; i < videoDirs.length; i++)
    {
      int exitCode = 0;
      if(isRootDirs)
      {
        Runtime runtime = Runtime.getRuntime();
        String[] cmd = new String[4];
        cmd[0] = "cmd.exe";
        cmd[1] = "/C";
        cmd[2] = "dir";
        cmd[3] = videoDirs[i].getAbsolutePath();
        Process process = runtime.exec(cmd);
        process.getOutputStream();
        StreamGobbler errorGobbler = new StreamGobbler(process.getErrorStream(), "ERROR");
        StreamGobbler outputGobbler = new StreamGobbler(process.getInputStream(), "OUTPUT");
        errorGobbler.logOutput = false;
        outputGobbler.logOutput = false;
        errorGobbler.start();
        outputGobbler.start();
        try{
          exitCode = process.waitFor();
        }
        catch(InterruptedException ie)
        {
          exitCode = -1;
        }
      }
      if(exitCode == 0)
      {
        writer.println("<input type=\"checkbox\" name=\"directory" + i + "\" value=\"" + videoDirs[i].getAbsolutePath() + "\"/>");
        File[] subDirs = videoDirs[i].listFiles(new DirFilter());
        if(subDirs != null && subDirs.length > 0)
        {
          writer.print("<a href=\"SelectSubDirectory?index=" + videoDirs[i].getAbsolutePath() + "\">");
          writer.print(getDirName(videoDirs[i].getPath()));
          writer.println("</a>");
        }
        else
        {
          writer.print(getDirName(videoDirs[i].getPath()));
        }
        writer.println("<br/>");
      }
    }
    writer.println("<br/>");
    writer.println("<button type=\"submit\" name=\"dirButton\" value=\"Add Video Dir(s)\">Add Video Dir(s)</button>");
    
    writer.println("</td></tr></table>");
    
    writer.println("</form>");
    writer.println("<hr/>");
    writer.println("<a href=\"configFileExtensions\"><strong>Next (File Extensions)</strong></a>");
    writer.println("<br/><br/>");
    writer.println("<strong>Note: This is the only configuration option you are *required* to complete." +
        " You can click \"Finish\" below, or click \"Next\" above to configure other options.</strong>");


    printFooterHtml(writer);
    writer.flush();

  }

  private void writeColor(String color, PrintWriter writer, String html)
  {
    writer.println("<font color=\"" + color + "\">" + html + "</font>");
  }
  
  private String getDirName(String dir)
  {
    if(dir.length() < 4)
    {
      return dir;
    }
    
    String dirname = dir.substring(dir.lastIndexOf("\\"), dir.length());
    return dirname;
  }
  
  private void renderFinish(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    PrintWriter writer = response.getWriter();
    printHeaderHtml(writer);
    //Print errors
    if(request.getAttribute("error") != null)
    {
      writer.println("<font color=\"#FF7777\">" + request.getAttribute("error") + "</font><br/>");
    }

    //Video Dirs
    writer.println("Comskip Monitor successfully configured.<br/><br/>");
    writer.println("Please close this browser window.");

    writer.println("</body>");
    writer.println("</html>");
    writer.flush();

  }

  private void printTopPage(PrintWriter writer)
  {
    writer.println("<h2>Comskip Monitor Configuration</h2><hr/>");
  }

  private void renderConfigFileExtensions(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    PrintWriter writer = response.getWriter();
    printHeaderHtml(writer);
    //Print errors
    if(request.getAttribute("error") != null)
    {
      writer.println("<font color=\"#FF7777\">" + request.getAttribute("error") + "</font><br/>");
    }

    writer.println("<form name=\"vidForm\" action=\"AddProcessExtension\" method=\"post\">");
    writer.println("<strong>Extensions of files to process:</strong><br/>");
    writer.println("<em>Please enter the file extensions for the type of files you would " +
        "like Comskip to process.  <br/>Example: .mpg, .mpeg <br/>Note:  You are not required to change" +
        " the default values.</em><br/><br/>");


    String extProp = props.getProperty("process.file.extensions");

    StringTokenizer tokens = new StringTokenizer(extProp, ",");
    if(tokens.countTokens() <= 0)
    {
      writeColor(RED, writer, "<strong>None Selected.</strong><br/>");
    }
    int index = 1;
    while (tokens.hasMoreTokens())
    {
      writeColor(GREEN, writer, "<strong>" + index + ") " + tokens.nextToken() + "</strong> &nbsp;&nbsp;&nbsp;<a href=\"RemoveProcessExtension?index="
          + index  + "\">Remove</a><br/>");
      index++;
    }
    writer.println("<input type=\"text\" name=\"ext\"/> <button type=\"submit\" name=\"dirButton\" value=\"Add Directory\">Add File Extension</button>");
    writer.println("</form>");

    writer.println("<hr/>");
    writer.print("<a href=\"configVideoDirs\">Previous (Video Directories)</a>");
    writer.print(" - ");
    writer.println("<a href=\"configCleanupExtensions\"><strong>Next (Cleanup Extensions)</strong></a>");

    printFooterHtml(writer);
    writer.flush();

  }

  private void renderConfigCleanupExtensions(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    PrintWriter writer = response.getWriter();
    printHeaderHtml(writer);
    //Print errors
    if(request.getAttribute("error") != null)
    {
      writer.println("<font color=\"#FF7777\">" + request.getAttribute("error") + "</font><br/>");
    }

    writer.println("<form name=\"vidForm\" action=\"AddCleanupExtension\" method=\"post\">");
    writer.println("<strong>Extensions of files to clean up:</strong><br/>");
    writer.println("<em>Please enter the file extensions for the type of files you would " +
        "like Comskip Monitor to cleanup once there is no related video file.  " +
        "Examples: .txt, .edl <br/>Note:  You are not required to change" +
        " the default values.</em><br/><br/>");


    String extProp = props.getProperty("cleanup.file.extensions");
    StringTokenizer tokens = new StringTokenizer(extProp, ",");
    if(tokens.countTokens() <= 0)
    {
      writeColor(RED, writer, "<strong>None Selected.</strong><br/>");
    }
    int index = 1;
    while (tokens.hasMoreTokens())
    {
      writeColor(GREEN, writer, "<strong>" + index + ") " + tokens.nextToken() + " </strong> &nbsp;&nbsp;&nbsp;<a href=\"RemoveCleanupExtension?index="
          + index  + "\">Remove</a><br/>");
      index++;
    }
    writer.println("<input type=\"text\" name=\"ext\"/> <button type=\"submit\" name=\"dirButton\" value=\"Add Directory\">Add File Extension</button>");
    writer.println("</form>");

    writer.println("<font color=\"#228822\"><form name=\"vidForm\" action=\"AddRelatedFileExtension\" method=\"post\">");
    writer.println("<strong>*Advanced: Related video file extensions.*</strong><br/>");
    writer.println("<em>Enter file extensions for additinal file types that, if present, will " +
        "prevent Comskip Monitor from removing the log files specified above. <br/>Note: This information " +
        "is not required. <br/>Example:  If you convert an .mpg file to an .avi file and then delete the .mpg file, " +
        "you would add the .avi file extension here. " +
        " </em><br/><br/></font>");

    extProp = props.getProperty("related.file.extensions");
    tokens = new StringTokenizer(extProp, ",");
    if(tokens.countTokens() <= 0)
    {
      writer.println("<strong>None Selected.</strong><br/>");
    }
    index = 1;
    while (tokens.hasMoreTokens())
    {
      writer.println("<strong>" + index + ") " + tokens.nextToken() + "</strong> &nbsp;&nbsp;&nbsp;<a href=\"RemoveRelatedExtension?index="
          + index  + "\">Remove</a><br/>");
      index++;
    }
    writer.println("<input type=\"text\" name=\"ext\"/> <button type=\"submit\" name=\"dirButton\" value=\"Add Directory\">Add File Extension</button>");
    writer.println("</form>");

    writer.println("<hr/>");
    writer.print("<a href=\"configFileExtensions\">Previous (File Extensions)</a>");
    writer.print(" - ");
    writer.println("<a href=\"configComskipLocation\"><strong>Next (Comskip Location)</strong></a>");

    printFooterHtml(writer);
    writer.flush();

  }

  private void renderConfigComskipLocation(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    PrintWriter writer = response.getWriter();
    printHeaderHtml(writer);
    //Print errors
    if(request.getAttribute("error") != null)
    {
      writer.println("<font color=\"#FF7777\">" + request.getAttribute("error") + "</font><br/>");
    }

    writer.println("<form name=\"vidForm\" action=\"SetComskipLocation\" method=\"post\">");
    writer.println("<strong>Set Comskip Location:</strong><br/>");
    writer.println("<em>Please select the location of your Comskip executable file.  <br/>Note 1: " +
        " Comskip Monitor comes with Comskip and this value does not require changing unless " +
        "you already have another Comskip installation you would like to use." +
        "<br/>Note 2: It is possible to select a program other than Comskip to process your files.</em><br/><br/>");

    String prop = props.getProperty("comskip.location").trim();
    if(prop.equals(""))
    {
      writeColor(RED, writer, "Not Selected.<br/>");
    }
    else
    {
      writeColor(GREEN, writer, "<strong>" + prop + "</strong>" );
    }
    writer.println("<br/>");
    writer.println("<input type=\"hidden\" name=\"realComskipLoc\" />");
    writer.println("<input type=\"file\" name=\"comskipLoc\" " +
        "onchange=\"document.vidForm.realComskipLoc.value=document.vidForm.comskipLoc.value;submit()\"/>");
    writer.println("<br/>");
    writer.println("</form>");

    writer.println("<font color=\"#228822\"><form name=\"cmdForm\" action=\"AddComskipParam\" method=\"post\">");
    writer.println("<strong>*Advanced: Comskip command line parameters.*</strong><br/>");
    writer.println("<em>Enter any additional command line parameters you would like comskip to execute.<br/>Note 1: This information " +
        "is not required, and bad parameters will cause Comskip not to execute.<br/>Note 2: The command line " +
        "option -n (--playnice) should be set from the 'Configure" +
        " Exclusion Hours' section by selecing the 'Run Slow' option.  <br/>Example: --videoredo (Outputs a VideoRedo cutlist)" +
        " </em><br/><br/></font>");

    String paramProp = props.getProperty("comskip.parameters");
    StringTokenizer tokens = new StringTokenizer(paramProp, ",");
    if(tokens.countTokens() <= 0)
    {
      writer.println("<strong>None Entered.</strong><br/>");
    }
    int index = 1;
    while (tokens.hasMoreTokens())
    {
      writer.println("<strong>" + index + ") " + tokens.nextToken() + "</strong> &nbsp;&nbsp;&nbsp;<a href=\"RemoveComskipParam?index="
          + index  + "\">Remove</a><br/>");
      index++;
    }
    writer.println("<input type=\"text\" name=\"param\"/> <button type=\"submit\" name=\"dirButton\" value=\"Add Directory\">Add Parameter</button>");
    writer.println("</form>");
    
    
    
    
    writer.println("<hr/>");
    writer.print("<a href=\"configCleanupExtensions\">Previous (Cleanup Extensions)</a>");
    writer.print(" - ");
    writer.println("<a href=\"configComskipConfigLocation\"><strong>Next (Comskip Config Location)</strong></a>");
    printFooterHtml(writer);
    writer.flush();

  }

  private void renderConfigConfigLocation(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    PrintWriter writer = response.getWriter();
    printHeaderHtml(writer);
    //Print errors
    if(request.getAttribute("error") != null)
    {
      writer.println("<font color=\"#FF7777\">" + request.getAttribute("error") + "</font><br/>");
    }

    String comskipLoc = props.getProperty("comskip.location").trim();
    if(comskipLoc.endsWith("comskip.exe"))
    {
      writer.println("<form name=\"vidForm\" action=\"SetConfigLocation\" method=\"post\">");
      writer.println("<strong>Set Comskip Config Location:</strong><br/>");
      writer.println("<em>Please select the location of your Comskip config file \"comskip.ini\".  <br/>Note: " +
          " Comskip Monitor comes with Comskip and this value does not require changing unless " +
          "you already have another Comskip config file you would like to use.</em><br/><br/>");

      String prop = props.getProperty("comskip.config").trim();
      if(prop.equals(""))
      {
        writeColor(RED, writer, "Not Selected.<br/>");
      }
      else
      {
        writeColor(GREEN, writer, "<strong>" + prop + "</strong>" );
      }
      writer.println("<br/>");
      writer.println("<input type=\"hidden\" name=\"realConfigLoc\" />");
      writer.println("<input type=\"file\" name=\"configLoc\" " +
      "onchange=\"document.vidForm.realConfigLoc.value=document.vidForm.configLoc.value;submit()\"/>");
      writer.println("<br/>");
      writer.println("</form>");
    }
    else
    {
      writeColor(GREEN, writer, "<strong>You've selected a program other than Comskip on the " +
          "'Configure Comskip Location' screen.  Therefore, you do not need to specify a Comskip config " +
          "file.</strong>");
    }
    
    writer.println("<hr/>");
    writer.print("<a href=\"configComskipLocation\">Previous (Comskip Location)</a>");
    writer.print(" - ");
    writer.println("<a href=\"configCheckTime\"><strong>Next (Check time interval)</strong></a>");

    printFooterHtml(writer);
    writer.flush();
  }

  private void renderConfigCheckTime(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    PrintWriter writer = response.getWriter();
    printHeaderHtml(writer);
    //Print errors
    if(request.getAttribute("error") != null)
    {
      writer.println("<font color=\"#FF7777\">" + request.getAttribute("error") + "</font><br/>");
    }

    writer.println("<form name=\"vidForm\" action=\"SetCheckTime\" method=\"post\">");
    writer.println("<strong>Set time between checks for new files:</strong><br/>");
    writer.println("<em>This is the time, in minutes, between when Comskip Monitor checks for " +
        "the existence of new video files.  Comskip Monitor will not check for new video files " +
        "while Comskip is processing.  <br/>Note: The default value is not required to be changed." +
        "</em><br/><br/>");

    String prop = props.getProperty("time.between.checks").trim();
    String continuous = props.getProperty("execute.during.record");
    String checked = "";
    if(continuous != null && continuous.equals("yes"))
    {
      checked = " checked=\"checked\" ";
    }

    if(prop.equals(""))
    {
      writeColor(RED, writer, "<strong>Not Set.</strong><br/>");
    }
    else
    {
      writeColor(GREEN, writer, "<strong>Current Setting: " + prop + " minutes</strong>");
    }
    writer.println("<br/>");
    writer.println("<input type=\"text\" name=\"checkTime\"/> (minutes)");
    writer.println("<br/>");
    writer.println("<br/>");
    writer.println("<input type=\"checkbox\" name=\"continuous\" value=\"true\" " + checked + "/> " +
        "Check this box if you would like Comskip to process video files as they are being recorded.  Note: " +
        "You are not required to change this value.");
    writer.println("<br/><br/>");
    writer.println("<button type=\"submit\" name=\"dirButton\" value=\"Add Directory\">Set Time Options</button>");
    writer.println("</form>");
    writer.println("<hr/>");
    writer.print("<a href=\"configComskipConfigLocation\">Previous (Comskip config location)</a>");
    writer.print(" - ");
    writer.println("<a href=\"configExcludeHours\"><strong>Next (Exclude hours)</strong></a>");

    printFooterHtml(writer);
    writer.flush();

  }

  private void renderConfigExcludeHours(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    PrintWriter writer = response.getWriter();
    printHeaderHtml(writer);
    //Print errors
    if(request.getAttribute("error") != null)
    {
      writer.println("<font color=\"#FF7777\">" + request.getAttribute("error") + "</font><br/>");
    }

    writer.println("<form name=\"vidForm\" action=\"SetExcludeHours\" method=\"post\">");
    writer.println("<strong>Set hours when Comskip should not run:</strong><br/>");
    writer.println("<em>If there are certain times of the day when you would prefer that Comskip Monitor " +
        "not process Comskip on new video files.  You can select those hours below.</em>" +
        "<br/><br/>-Run Normal: Comskip will process files normally.<br/>" +
        "-Run Slow: Comskip will process files using extra low priority. " +
        "<br/>-Don't Run: Comskip will not process files.<br/>");

    String prop = props.getProperty("exclude.hours").trim();
    StringTokenizer tokens = new StringTokenizer(prop, ",");
    List hourList = new ArrayList();
    while (tokens.hasMoreTokens())
    {
      hourList.add(tokens.nextToken().trim());
    }

    prop = props.getProperty("slow.hours").trim();
    tokens = new StringTokenizer(prop, ",");
    List slowHourList = new ArrayList();
    while (tokens.hasMoreTokens())
    {
      slowHourList.add(tokens.nextToken().trim());
    }
    
    writer.println("<table border=\"0\" cellpadding=\"15\"><tr><td>");
    
    for (int i = 0; i < 24; i++)
    {
      int startTime = i;
      String startampm = " AM";
      int endTime = i+1;
      String endampm = " AM";
      if(startTime >= 12)
      {
        startampm = " PM";
        startTime -= 12;
      }
      if(endTime >= 12)
      {
        endampm = " PM";
        endTime -= 12;
      }
      if(startTime == 0)
      {
        startTime = 12;
      }
      if(endTime == 0)
      {
        endTime = 12;
      }

      String onChecked = "";
      String slowChecked = "";
      String offChecked = "";
      if(hourList.contains(String.valueOf(i)))
      {
        offChecked = " checked=\"checked\"";
      }
      else if(slowHourList.contains(String.valueOf(i)))
      {
        slowChecked = " checked=\"checked\"";
      }
      else
      {
        onChecked = " checked=\"checked\"";
      }

      if(i % 2 == 0)
      {
        writer.print("<font color=\"" + BLUE + "\"> ");
      }
      else
      {
        writer.print("<font color=\"" + GREEN + "\"> ");
      }
      
      writer.println("<input type=\"radio\" name=\"time_" + i + "\" value=\"on\"" + onChecked + "/> Run Normal " +
          "<input type=\"radio\" name=\"time_" + i + "\" value=\"slow\"" + slowChecked + "/> Run Slow " +
          "<input type=\"radio\" name=\"time_" + i + "\" value=\"off\"" + offChecked + "/> Don't Run | <strong>" +  startTime + startampm + " - " + endTime + endampm + "</strong><br/>");

      writer.print("</font>");

      if(i == 11)
      {
        writer.println("</td><td>");
      }
    }
    writer.println("</tr></table>");

    writer.println("<button type=\"submit\" name=\"dirButton\" value=\"Add Directory\">Set Hours</button> ");
    writeColor(RED, writer, "<-- Click this button before clicking 'Finish' in order to save this information.");
    writer.println("</form>");

    writer.println("<hr/>");
    writer.print("<a href=\"configCheckTime\">Previous (Check time interval)</a>");
    writer.print(" - ");
    writer.println("<a href=\"finish\"><strong>Finish</strong></a>");
    printFooterHtml(writer);
    writer.flush();

  }


  protected void doPost(HttpServletRequest request, HttpServletResponse response)
      throws ServletException, IOException
  {
    
    if(request.getPathInfo().equals("/AddVideoDir"))
    {
      String videoDirsProp = props.getProperty("file.directories");
      List dirList = new ArrayList();
      StringTokenizer tokenizer = new StringTokenizer(videoDirsProp, ",");
      while(tokenizer.hasMoreTokens())
      {
        dirList.add(tokenizer.nextToken().trim());
      }
      
      Enumeration dirEnum = request.getParameterNames();
      boolean isFirst = true;
      if(videoDirsProp != null && videoDirsProp.trim().length() > 2)
      {
        isFirst = false;
      }
      while(dirEnum.hasMoreElements())
      {
        String paramName = (String)dirEnum.nextElement();
        if(paramName.startsWith("directory"))
        {
          String value = request.getParameter(paramName);
          if(dirList.contains(value))
          {
            request.setAttribute("error", "One or more directories not added because " +
                "it was already selected.");
          }
          else
          {
            if(isFirst)
            {
              isFirst = false;
            }
            else
            {
              videoDirsProp += ", ";
            }
            videoDirsProp += value;
          }
        }
      }
      
      props.setProperty("file.directories", videoDirsProp);
      renderConfigVideoDirs(request, response);
    }

    if(request.getPathInfo().equals("/AddProcessExtension"))
    {
      String newExt = request.getParameter("ext");
      if(!newExt.trim().equals(""))
      {
        if(!newExt.trim().startsWith("."))
        {
          newExt = "." + newExt;
        }
        String extProp = props.getProperty("process.file.extensions");
        String comma = "";
        if(extProp.trim().length() > 0)
        {
          comma = ", ";
        }
        props.setProperty("process.file.extensions", (extProp += comma + newExt));
        System.out.println("extension added");
      }
      renderConfigFileExtensions(request, response);
    }

    if(request.getPathInfo().equals("/AddCleanupExtension"))
    {
      String newExt = request.getParameter("ext");
      if(!newExt.trim().equals(""))
      {
        if(!newExt.trim().startsWith("."))
        {
          newExt = "." + newExt;
        }
        String extProp = props.getProperty("cleanup.file.extensions");
        String comma = "";
        if(extProp.trim().length() > 0)
        {
          comma = ", ";
        }
        props.setProperty("cleanup.file.extensions", (extProp += comma + newExt));
        System.out.println("extension added");
      }
      renderConfigCleanupExtensions(request, response);
    }
    
    if(request.getPathInfo().equals("/AddComskipParam"))
    {
      String newParam = request.getParameter("param");
      if(!newParam.trim().equals(""))
      {
        String params = props.getProperty("comskip.parameters");
        String comma = "";
        if(params.trim().length() > 0)
        {
          comma = ", ";
        }
        props.setProperty("comskip.parameters", (params += comma + newParam));
        System.out.println("parameter added");
      }
      renderConfigComskipLocation(request, response);
    }

    if(request.getPathInfo().equals("/AddRelatedFileExtension"))
    {
      String newExt = request.getParameter("ext");
      if(!newExt.trim().equals(""))
      {
        if(!newExt.trim().startsWith("."))
        {
          newExt = "." + newExt;
        }
        String extProp = props.getProperty("related.file.extensions");
        String comma = "";
        if(extProp.trim().length() > 0)
        {
          comma = ", ";
        }
        props.setProperty("related.file.extensions", (extProp += comma + newExt));
        System.out.println("extension added");
      }
      renderConfigCleanupExtensions(request, response);
    }

    if(request.getPathInfo().equals("/SetComskipLocation"))
    {
      String comskipLoc = request.getParameter("realComskipLoc");
      File tempFile = new File(comskipLoc);
      if(tempFile.exists())
      {
        props.setProperty("comskip.location", tempFile.getAbsolutePath());
        System.out.println("comskip location set");
      }
      else
      {
        request.setAttribute("error", "Invalid Comskip File");
        System.out.println("invalid comskip file");
      }
      renderConfigComskipLocation(request, response);
    }

    if(request.getPathInfo().equals("/SetConfigLocation"))
    {
      String configLoc = request.getParameter("realConfigLoc");
      File tempFile = new File(configLoc);
      if(tempFile.exists())
      {
        props.setProperty("comskip.config", tempFile.getAbsolutePath());
        System.out.println("comskip config location set");
      }
      else
      {
        request.setAttribute("error", "Invalid Comskip Config File");
        System.out.println("invalid comskip config file");
      }
      renderConfigConfigLocation(request, response);
    }

    if(request.getPathInfo().equals("/SetCheckTime"))
    {
      String checkTime = request.getParameter("checkTime");
      String continuous = request.getParameter("continuous");
      try
      {
        System.out.println(continuous);
        if(continuous != null && continuous.equals("true"))
        {
          props.setProperty("execute.during.record", "yes");
        }
        else
        {
          props.setProperty("execute.during.record", "no");
        }
        
        if(checkTime != null && checkTime.trim().length() > 0)
        {
          int checkTimeInt = Integer.parseInt(checkTime.trim());
          props.setProperty("time.between.checks", checkTime.trim());
          System.out.println("CheckTime set");
        }
      }
      catch(Exception e)
      {
        request.setAttribute("error", "Invalid number.  Please enter a number.");
      }
      renderConfigCheckTime(request, response);
    }

    if(request.getPathInfo().equals("/SetExcludeHours"))
    {
      String excludeHours = "";
      String slowHours = "";
      Enumeration timeEnum = request.getParameterNames();
      boolean isFirstExclude = true;
      boolean isFirstSlow = true;
      while(timeEnum.hasMoreElements())
      {
        String paramName = (String)timeEnum.nextElement();
        if(paramName.startsWith("time_"))
        {
          String hour = paramName.substring(paramName.indexOf("_") + 1, paramName.length());
          String value = request.getParameter(paramName);
          if(value.equals("off"))
          {
            if(isFirstExclude)
            {
              isFirstExclude = false;
            }
            else
            {
              excludeHours += ", ";
            }
            excludeHours += hour;
          }
          else if(value.equals("slow"))
          {
            if(isFirstSlow)
            {
              isFirstSlow = false;
            }
            else
            {
              slowHours += ", ";
            }
            slowHours += hour;
          }
        }
      }
      props.setProperty("exclude.hours", excludeHours);
      props.setProperty("slow.hours", slowHours);
      System.out.println("Exclude hours set");
      renderConfigExcludeHours(request, response);
    }

  }

  private void printHeaderHtml(PrintWriter writer)
  {
    writer.println("<html>");
    writer.println("<title>Comskip Monitor Admin</title>");
    writer.println("<body>");
    printTopPage(writer);
  }

  private void printFooterHtml(PrintWriter writer)
  {
    renderBottomNavOptions(writer);
    writer.println("</body>");
    writer.println("</html>");
  }

  /**
   * Loads the properties file from the file csm.properties
   * which should exist in the same directory as ComSkipMonitor
   *
   */
  private Properties loadProperties(String filename)
  {
    Properties props = new Properties();
    try
    {
      File propFile = new File(filename);
      InputStream inStream = new FileInputStream(propFile);
      props.load(inStream);
      return props;
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("Unable to load " + filename + " file, please ensure this file"
          + "exists");
    }
    catch (IOException ioe)
    {
      System.out.println("Unable to read " + filename + " file.");
    }
    return props;
  }

  
  /**
   * Loads the properties file from the file csm.properties
   * which should exist in the same directory as ComSkipMonitor
   *
   */
  private void savePropertiesViaWriter(Properties props, String filename)
  {
    try
    {
      PrintWriter out = new PrintWriter(new FileWriter(filename));
      Enumeration keyEnum = props.keys();
      while(keyEnum.hasMoreElements())
      {
        String key = (String)keyEnum.nextElement();
        String val = props.getProperty(key);
        out.println(key + "=" + val);
      }
      out.flush();
      out.close();
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("Unable to save " + filename + " file.");
    }
    catch (IOException ioe)
    {
      System.out.println("Unable to save " + filename + " file.");
    }
  }

  /**
   * Loads the properties file from the file csm.properties
   * which should exist in the same directory as ComSkipMonitor
   *
   */
  private void saveProperties(Properties props, String filename)
  {
    try
    {
      File propFile = new File(filename);
      OutputStream outStream = new FileOutputStream(propFile);
      props.store(outStream, filename);
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("Unable to save " + filename + " file.");
    }
    catch (IOException ioe)
    {
      System.out.println("Unable to save csm.properties file.");
    }
  }

  /**
   * Display an error message to the user by sending an html
   * message file to the user.
   *
   */
  private void alert(String messageFile)
  {
    Runtime runtime = Runtime.getRuntime();
    File invalidDirectoryMessage = new File(messageFile);
    String[] cmd = new String[4];
    cmd[0] = "cmd.exe";
    cmd[1] = "/C";
    cmd[2] = "start";
    cmd[3] = invalidDirectoryMessage.getAbsolutePath();
    try
    {
      runtime.exec(cmd).waitFor();
    }
    catch (IOException ioe)
    {}
    catch (InterruptedException ie)
    {}
  }

  private void renderBottomNavOptions(PrintWriter writer)
  {
    writer.println("<hr/>");
    writer.println("<a href=\"finish\"><strong>Finish Configuration</strong></a><br/>");
    writer.println("<a href=\"configVideoDirs\">Configure Video Directories</a><br/>");
    writer.println("<a href=\"configFileExtensions\">Configure File Extensions</a><br/>");
    writer.println("<a href=\"configCleanupExtensions\">Configure Cleanup Extensions</a><br/>");
    writer.println("<a href=\"configComskipLocation\">Configure Comskip Location</a><br/>");
    writer.println("<a href=\"configComskipConfigLocation\">Configure Comskip Config Location</a><br/>");
    writer.println("<a href=\"configCheckTime\">Configure Time Between Checks</a><br/>");
    writer.println("<a href=\"configExcludeHours\">Configure Exclusion Hours</a><br/>");
  }

}

class DirFilter implements FileFilter{

  public boolean accept(File pathname)
  {
    return pathname.isDirectory();
  }
}